@@ -71,7 +71,6 @@ static int qemu_system_ready;
 #define SIG_IPI (SIGRTMIN+4)
 
 pthread_t io_thread;
-static int io_thread_fd = -1;
 static int io_thread_sigfd = -1;
 
 static CPUState *kvm_debug_cpu_requested;
@@ -147,7 +146,7 @@ static inline void clear_gsi(kvm_context_t kvm, unsigned int gsi)
 
 static int kvm_create_context(void);
 
-int kvm_init(int smp_cpus)
+int kvm_init(void)
 {
     int fd;
     int r, gsi_count;
@@ -201,8 +200,9 @@ int kvm_init(int smp_cpus)
         kvm_context->max_gsi = gsi_bits;
 
         /* Mark any over-allocated bits as already in use */
-        for (i = gsi_count; i < gsi_bits; i++)
+        for (i = gsi_count; i < gsi_bits; i++) {
             set_gsi(kvm_context, i);
+        }
     }
 
     kvm_cpu_register_phys_memory_client();
@@ -296,8 +296,9 @@ static int kvm_set_boot_vcpu_id(kvm_context_t kvm, uint32_t id)
 {
 #ifdef KVM_CAP_SET_BOOT_CPU_ID
     int r = kvm_ioctl(kvm_state, KVM_CHECK_EXTENSION, KVM_CAP_SET_BOOT_CPU_ID);
-    if (r > 0)
+    if (r > 0) {
         return kvm_vm_ioctl(kvm_state, KVM_SET_BOOT_CPU_ID, id);
+    }
     return -ENOSYS;
 #else
     return -ENOSYS;
@@ -352,8 +353,9 @@ void kvm_create_irqchip(kvm_context_t kvm)
 #if defined(KVM_CAP_IRQ_INJECT_STATUS) && defined(KVM_IRQ_LINE_STATUS)
                 r = kvm_ioctl(kvm_state, KVM_CHECK_EXTENSION,
                               KVM_CAP_IRQ_INJECT_STATUS);
-                if (r > 0)
+                if (r > 0) {
                     kvm->irqchip_inject_ioctl = KVM_IRQ_LINE_STATUS;
+                }
 #endif
                 kvm->irqchip_in_kernel = 1;
             } else
@@ -369,17 +371,22 @@ int kvm_create(kvm_context_t kvm, unsigned long phys_mem_bytes, void **vm_mem)
     int r, i;
 
     r = kvm_create_vm(kvm);
-    if (r < 0)
+    if (r < 0) {
         return r;
+    }
     r = kvm_arch_create(kvm, phys_mem_bytes, vm_mem);
-    if (r < 0)
+    if (r < 0) {
         return r;
-    for (i = 0; i < ARRAY_SIZE(kvm_state->slots); i++)
+    }
+    for (i = 0; i < ARRAY_SIZE(kvm_state->slots); i++) {
         kvm_state->slots[i].slot = i;
+    }
 
     r = kvm_create_default_phys_mem(kvm, phys_mem_bytes, vm_mem);
-    if (r < 0)
+    if (r < 0) {
         return r;
+    }
+
     kvm_create_irqchip(kvm);
 
     return 0;
@@ -392,13 +399,15 @@ int kvm_set_irq_level(kvm_context_t kvm, int irq, int level, int *status)
     struct kvm_irq_level event;
     int r;
 
-    if (!kvm->irqchip_in_kernel)
+    if (!kvm->irqchip_in_kernel) {
         return 0;
+    }
     event.level = level;
     event.irq = irq;
     r = kvm_vm_ioctl(kvm_state, kvm->irqchip_inject_ioctl, &event);
-    if (r < 0)
+    if (r < 0) {
         perror("kvm_set_irq_level");
+    }
 
     if (status) {
 #ifdef KVM_CAP_IRQ_INJECT_STATUS
@@ -416,8 +425,9 @@ int kvm_get_irqchip(kvm_context_t kvm, struct kvm_irqchip *chip)
 {
     int r;
 
-    if (!kvm->irqchip_in_kernel)
+    if (!kvm->irqchip_in_kernel) {
         return 0;
+    }
     r = kvm_vm_ioctl(kvm_state, KVM_GET_IRQCHIP, chip);
     if (r < 0) {
         perror("kvm_get_irqchip\n");
@@ -429,8 +439,9 @@ int kvm_set_irqchip(kvm_context_t kvm, struct kvm_irqchip *chip)
 {
     int r;
 
-    if (!kvm->irqchip_in_kernel)
+    if (!kvm->irqchip_in_kernel) {
         return 0;
+    }
     r = kvm_vm_ioctl(kvm_state, KVM_SET_IRQCHIP, chip);
     if (r < 0) {
         perror("kvm_set_irqchip\n");
@@ -461,34 +472,15 @@ int kvm_set_regs(CPUState *env, struct kvm_regs *regs)
     return kvm_vcpu_ioctl(env, KVM_SET_REGS, regs);
 }
 
-int kvm_get_fpu(CPUState *env, struct kvm_fpu *fpu)
-{
-    return kvm_vcpu_ioctl(env, KVM_GET_FPU, fpu);
-}
-
-int kvm_set_fpu(CPUState *env, struct kvm_fpu *fpu)
-{
-    return kvm_vcpu_ioctl(env, KVM_SET_FPU, fpu);
-}
-
-int kvm_get_sregs(CPUState *env, struct kvm_sregs *sregs)
-{
-    return kvm_vcpu_ioctl(env, KVM_GET_SREGS, sregs);
-}
-
-int kvm_set_sregs(CPUState *env, struct kvm_sregs *sregs)
-{
-    return kvm_vcpu_ioctl(env, KVM_SET_SREGS, sregs);
-}
-
 #ifdef KVM_CAP_MP_STATE
 int kvm_get_mpstate(CPUState *env, struct kvm_mp_state *mp_state)
 {
     int r;
 
     r = kvm_ioctl(kvm_state, KVM_CHECK_EXTENSION, KVM_CAP_MP_STATE);
-    if (r > 0)
+    if (r > 0) {
         return kvm_vcpu_ioctl(env, KVM_GET_MP_STATE, mp_state);
+    }
     return -ENOSYS;
 }
 
@@ -497,36 +489,13 @@ int kvm_set_mpstate(CPUState *env, struct kvm_mp_state *mp_state)
     int r;
 
     r = kvm_ioctl(kvm_state, KVM_CHECK_EXTENSION, KVM_CAP_MP_STATE);
-    if (r > 0)
+    if (r > 0) {
         return kvm_vcpu_ioctl(env, KVM_SET_MP_STATE, mp_state);
+    }
     return -ENOSYS;
 }
 #endif
 
-#ifdef KVM_CAP_XSAVE
-int kvm_get_xsave(CPUState *env, struct kvm_xsave *xsave)
-{
-    return kvm_vcpu_ioctl(env, KVM_GET_XSAVE, xsave);
-}
-
-int kvm_set_xsave(CPUState *env, struct kvm_xsave *xsave)
-{
-    return kvm_vcpu_ioctl(env, KVM_SET_XSAVE, xsave);
-}
-#endif
-
-#ifdef KVM_CAP_XCRS
-int kvm_get_xcrs(CPUState *env, struct kvm_xcrs *xcrs)
-{
-    return kvm_vcpu_ioctl(env, KVM_GET_XCRS, xcrs);
-}
-
-int kvm_set_xcrs(CPUState *env, struct kvm_xcrs *xcrs)
-{
-    return kvm_vcpu_ioctl(env, KVM_SET_XCRS, xcrs);
-}
-#endif
-
 static int handle_mmio(CPUState *env)
 {
     unsigned long addr = env->kvm_run->mmio.phys_addr;
@@ -534,8 +503,9 @@ static int handle_mmio(CPUState *env)
     void *data = kvm_run->mmio.data;
 
     /* hack: Red Hat 7.1 generates these weird accesses. */
-    if ((addr > 0xa0000 - 4 && addr <= 0xa0000) && kvm_run->mmio.len == 3)
+    if ((addr > 0xa0000 - 4 && addr <= 0xa0000) && kvm_run->mmio.len == 3) {
         return 0;
+    }
 
     cpu_physical_memory_rw(addr, data, kvm_run->mmio.len, kvm_run->mmio.is_write);
     return 0;
@@ -596,13 +566,15 @@ int kvm_run(CPUState *env)
     }
     push_nmi(kvm);
 #if !defined(__s390__)
-    if (!kvm->irqchip_in_kernel)
+    if (!kvm->irqchip_in_kernel) {
         run->request_interrupt_window = kvm_arch_try_push_interrupts(env);
+    }
 #endif
 
     r = pre_kvm_run(kvm, env);
-    if (r)
+    if (r) {
         return r;
+    }
     if (env->exit_request) {
         env->exit_request = 0;
         pthread_kill(env->kvm_cpu_state.thread, SIG_IPI);
@@ -684,9 +656,10 @@ int kvm_run(CPUState *env)
             break;
         }
     }
-  more:
-    if (!r)
+more:
+    if (!r) {
         goto again;
+    }
     return r;
 }
 
@@ -783,7 +756,7 @@ int kvm_reinject_control(kvm_context_t kvm, int pit_reinject)
     return -ENOSYS;
 }
 
-int kvm_has_gsi_routing(kvm_context_t kvm)
+int kvm_has_gsi_routing(void)
 {
     int r = 0;
 
@@ -802,9 +775,11 @@ int kvm_get_gsi_count(kvm_context_t kvm)
 #endif
 }
 
-int kvm_clear_gsi_routes(kvm_context_t kvm)
+int kvm_clear_gsi_routes(void)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
+    kvm_context_t kvm = kvm_context;
+
     kvm->irq_routes->nr = 0;
     return 0;
 #else
@@ -812,23 +787,25 @@ int kvm_clear_gsi_routes(kvm_context_t kvm)
 #endif
 }
 
-int kvm_add_routing_entry(kvm_context_t kvm,
-                          struct kvm_irq_routing_entry *entry)
+int kvm_add_routing_entry(struct kvm_irq_routing_entry *entry)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
+    kvm_context_t kvm = kvm_context;
     struct kvm_irq_routing *z;
     struct kvm_irq_routing_entry *new;
     int n, size;
 
     if (kvm->irq_routes->nr == kvm->nr_allocated_irq_routes) {
         n = kvm->nr_allocated_irq_routes * 2;
-        if (n < 64)
+        if (n < 64) {
             n = 64;
+        }
         size = sizeof(struct kvm_irq_routing);
         size += n * sizeof(*new);
         z = realloc(kvm->irq_routes, size);
-        if (!z)
+        if (!z) {
             return -ENOMEM;
+        }
         kvm->nr_allocated_irq_routes = n;
         kvm->irq_routes = z;
     }
@@ -848,7 +825,7 @@ int kvm_add_routing_entry(kvm_context_t kvm,
 #endif
 }
 
-int kvm_add_irq_route(kvm_context_t kvm, int gsi, int irqchip, int pin)
+int kvm_add_irq_route(int gsi, int irqchip, int pin)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
     struct kvm_irq_routing_entry e;
@@ -858,16 +835,16 @@ int kvm_add_irq_route(kvm_context_t kvm, int gsi, int irqchip, int pin)
     e.flags = 0;
     e.u.irqchip.irqchip = irqchip;
     e.u.irqchip.pin = pin;
-    return kvm_add_routing_entry(kvm, &e);
+    return kvm_add_routing_entry(&e);
 #else
     return -ENOSYS;
 #endif
 }
 
-int kvm_del_routing_entry(kvm_context_t kvm,
-                          struct kvm_irq_routing_entry *entry)
+int kvm_del_routing_entry(struct kvm_irq_routing_entry *entry)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
+    kvm_context_t kvm = kvm_context;
     struct kvm_irq_routing_entry *e, *p;
     int i, gsi, found = 0;
 
@@ -910,8 +887,9 @@ int kvm_del_routing_entry(kvm_context_t kvm,
                     if (e->gsi == gsi)
                         break;
                 }
-                if (i == kvm->irq_routes->nr)
+                if (i == kvm->irq_routes->nr) {
                     clear_gsi(kvm, gsi);
+                }
 
                 return 0;
             }
@@ -923,11 +901,11 @@ int kvm_del_routing_entry(kvm_context_t kvm,
 #endif
 }
 
-int kvm_update_routing_entry(kvm_context_t kvm,
-                             struct kvm_irq_routing_entry *entry,
+int kvm_update_routing_entry(struct kvm_irq_routing_entry *entry,
                              struct kvm_irq_routing_entry *newentry)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
+    kvm_context_t kvm = kvm_context;
     struct kvm_irq_routing_entry *e;
     int i;
 
@@ -967,7 +945,7 @@ int kvm_update_routing_entry(kvm_context_t kvm,
 #endif
 }
 
-int kvm_del_irq_route(kvm_context_t kvm, int gsi, int irqchip, int pin)
+int kvm_del_irq_route(int gsi, int irqchip, int pin)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
     struct kvm_irq_routing_entry e;
@@ -977,15 +955,17 @@ int kvm_del_irq_route(kvm_context_t kvm, int gsi, int irqchip, int pin)
     e.flags = 0;
     e.u.irqchip.irqchip = irqchip;
     e.u.irqchip.pin = pin;
-    return kvm_del_routing_entry(kvm, &e);
+    return kvm_del_routing_entry(&e);
 #else
     return -ENOSYS;
 #endif
 }
 
-int kvm_commit_irq_routes(kvm_context_t kvm)
+int kvm_commit_irq_routes(void)
 {
 #ifdef KVM_CAP_IRQ_ROUTING
+    kvm_context_t kvm = kvm_context;
+
     kvm->irq_routes->flags = 0;
     return kvm_vm_ioctl(kvm_state, KVM_SET_GSI_ROUTING, kvm->irq_routes);
 #else
@@ -993,16 +973,18 @@ int kvm_commit_irq_routes(kvm_context_t kvm)
 #endif
 }
 
-int kvm_get_irq_route_gsi(kvm_context_t kvm)
+int kvm_get_irq_route_gsi(void)
 {
+    kvm_context_t kvm = kvm_context;
     int i, bit;
     uint32_t *buf = kvm->used_gsi_bitmap;
 
     /* Return the lowest unused GSI in the bitmap */
     for (i = 0; i < kvm->max_gsi / 32; i++) {
         bit = ffs(~buf[i]);
-        if (!bit)
+        if (!bit) {
             continue;
+        }
 
         return bit - 1 + i * 32;
     }
@@ -1010,6 +992,50 @@ int kvm_get_irq_route_gsi(kvm_context_t kvm)
     return -ENOSPC;
 }
 
+static void kvm_msix_routing_entry(struct kvm_irq_routing_entry *e,
+                                   uint32_t gsi, uint32_t addr_lo,
+                                   uint32_t addr_hi, uint32_t data)
+
+{
+    e->gsi = gsi;
+    e->type = KVM_IRQ_ROUTING_MSI;
+    e->flags = 0;
+    e->u.msi.address_lo = addr_lo;
+    e->u.msi.address_hi = addr_hi;
+    e->u.msi.data = data;
+}
+
+int kvm_add_msix(uint32_t gsi, uint32_t addr_lo,
+                        uint32_t addr_hi, uint32_t data)
+{
+    struct kvm_irq_routing_entry e;
+
+    kvm_msix_routing_entry(&e, gsi, addr_lo, addr_hi, data);
+    return kvm_add_routing_entry(&e);
+}
+
+int kvm_del_msix(uint32_t gsi, uint32_t addr_lo,
+                        uint32_t addr_hi, uint32_t data)
+{
+    struct kvm_irq_routing_entry e;
+
+    kvm_msix_routing_entry(&e, gsi, addr_lo, addr_hi, data);
+    return kvm_del_routing_entry(&e);
+}
+
+int kvm_update_msix(uint32_t old_gsi, uint32_t old_addr_lo,
+                    uint32_t old_addr_hi, uint32_t old_data,
+                    uint32_t new_gsi, uint32_t new_addr_lo,
+                    uint32_t new_addr_hi, uint32_t new_data)
+{
+    struct kvm_irq_routing_entry e1, e2;
+
+    kvm_msix_routing_entry(&e1, old_gsi, old_addr_lo, old_addr_hi, old_data);
+    kvm_msix_routing_entry(&e2, new_gsi, new_addr_lo, new_addr_hi, new_data);
+    return kvm_update_routing_entry(&e1, &e2);
+}
+
+
 #ifdef KVM_CAP_DEVICE_MSIX
 int kvm_assign_set_msix_nr(kvm_context_t kvm,
                            struct kvm_assigned_msix_nr *msix_nr)
@@ -1048,8 +1074,9 @@ int kvm_irqfd(kvm_context_t kvm, int gsi, int flags)
         return -ENOENT;
 
     fd = eventfd(0, 0);
-    if (fd < 0)
+    if (fd < 0) {
         return -errno;
+    }
 
     r = _kvm_irqfd(kvm, fd, gsi, 0);
     if (r < 0) {
@@ -1085,12 +1112,6 @@ static void sig_ipi_handler(int n)
 {
 }
 
-static void hardware_memory_error(void)
-{
-    fprintf(stderr, "Hardware memory error!\n");
-    exit(1);
-}
-
 static void sigbus_reraise(void)
 {
     sigset_t set;
@@ -1111,43 +1132,11 @@ static void sigbus_reraise(void)
 static void sigbus_handler(int n, struct qemu_signalfd_siginfo *siginfo,
                            void *ctx)
 {
-#if defined(KVM_CAP_MCE) && defined(TARGET_I386)
-    if (first_cpu->mcg_cap && siginfo->ssi_addr
-        && siginfo->ssi_code == BUS_MCEERR_AO) {
-        uint64_t status;
-        unsigned long paddr;
-        CPUState *cenv;
-
-        /* Hope we are lucky for AO MCE */
-        if (do_qemu_ram_addr_from_host((void *)(intptr_t)siginfo->ssi_addr,
-				       &paddr)) {
-            fprintf(stderr, "Hardware memory error for memory used by "
-                    "QEMU itself instead of guest system!: %llx\n",
-                    (unsigned long long)siginfo->ssi_addr);
-            return;
-        }
-        status = MCI_STATUS_VAL | MCI_STATUS_UC | MCI_STATUS_EN
-            | MCI_STATUS_MISCV | MCI_STATUS_ADDRV | MCI_STATUS_S
-            | 0xc0;
-        kvm_inject_x86_mce(first_cpu, 9, status,
-                           MCG_STATUS_MCIP | MCG_STATUS_RIPV, paddr,
-                           (MCM_ADDR_PHYS << 6) | 0xc, 1);
-        for (cenv = first_cpu->next_cpu; cenv != NULL; cenv = cenv->next_cpu)
-            kvm_inject_x86_mce(cenv, 1, MCI_STATUS_VAL | MCI_STATUS_UC,
-                               MCG_STATUS_MCIP | MCG_STATUS_RIPV, 0, 0, 1);
-    } else
-#endif
-    {
-        if (siginfo->ssi_code == BUS_MCEERR_AO)
-            return;
-        else if (siginfo->ssi_code == BUS_MCEERR_AR)
-            hardware_memory_error();
-        else
-            sigbus_reraise();
-    }
+    if (kvm_on_sigbus(siginfo->ssi_code, (void *)(intptr_t)siginfo->ssi_addr))
+        sigbus_reraise();
 }
 
-static void on_vcpu(CPUState *env, void (*func)(void *data), void *data)
+void on_vcpu(CPUState *env, void (*func)(void *data), void *data)
 {
     struct qemu_work_item wi;
 
@@ -1158,17 +1147,19 @@ static void on_vcpu(CPUState *env, void (*func)(void *data), void *data)
 
     wi.func = func;
     wi.data = data;
-    if (!env->kvm_cpu_state.queued_work_first)
+    if (!env->kvm_cpu_state.queued_work_first) {
         env->kvm_cpu_state.queued_work_first = &wi;
-    else
+    } else {
         env->kvm_cpu_state.queued_work_last->next = &wi;
+    }
     env->kvm_cpu_state.queued_work_last = &wi;
     wi.next = NULL;
     wi.done = false;
 
     pthread_kill(env->kvm_cpu_state.thread, SIG_IPI);
-    while (!wi.done)
+    while (!wi.done) {
         qemu_cond_wait(&qemu_work_cond);
+    }
 }
 
 static void do_kvm_cpu_synchronize_state(void *_env)
@@ -1183,8 +1174,9 @@ static void do_kvm_cpu_synchronize_state(void *_env)
 
 void kvm_cpu_synchronize_state(CPUState *env)
 {
-    if (!env->kvm_vcpu_dirty)
+    if (!env->kvm_vcpu_dirty) {
         on_vcpu(env, do_kvm_cpu_synchronize_state, env);
+    }
 }
 
 void kvm_cpu_synchronize_post_reset(CPUState *env)
@@ -1214,19 +1206,22 @@ void kvm_update_interrupt_request(CPUState *env)
     int signal = 0;
 
     if (env) {
-        if (!current_env || !current_env->created)
+        if (!current_env || !current_env->created) {
             signal = 1;
+        }
         /*
          * Testing for created here is really redundant
          */
         if (current_env && current_env->created &&
-            env != current_env && !env->kvm_cpu_state.signalled)
+            env != current_env && !env->kvm_cpu_state.signalled) {
             signal = 1;
+        }
 
         if (signal) {
             env->kvm_cpu_state.signalled = 1;
-            if (env->kvm_cpu_state.thread)
+            if (env->kvm_cpu_state.thread) {
                 pthread_kill(env->kvm_cpu_state.thread, SIG_IPI);
+            }
         }
     }
 }
@@ -1253,8 +1248,9 @@ static void flush_queued_work(CPUState *env)
 {
     struct qemu_work_item *wi;
 
-    if (!env->kvm_cpu_state.queued_work_first)
+    if (!env->kvm_cpu_state.queued_work_first) {
         return;
+    }
 
     while ((wi = env->kvm_cpu_state.queued_work_first)) {
         env->kvm_cpu_state.queued_work_first = wi->next;
@@ -1265,82 +1261,6 @@ static void flush_queued_work(CPUState *env)
     pthread_cond_broadcast(&qemu_work_cond);
 }
 
-static int kvm_mce_in_exception(CPUState *env)
-{
-    struct kvm_msr_entry msr_mcg_status = {
-        .index = MSR_MCG_STATUS,
-    };
-    int r;
-
-    r = kvm_get_msrs(env, &msr_mcg_status, 1);
-    if (r == -1 || r == 0)
-        return -1;
-    return !!(msr_mcg_status.data & MCG_STATUS_MCIP);
-}
-
-static void kvm_on_sigbus(CPUState *env, siginfo_t *siginfo)
-{
-#if defined(KVM_CAP_MCE) && defined(TARGET_I386)
-    struct kvm_x86_mce mce = {
-            .bank = 9,
-    };
-    unsigned long paddr;
-    int r;
-
-    if (env->mcg_cap && siginfo->si_addr
-        && (siginfo->si_code == BUS_MCEERR_AR
-            || siginfo->si_code == BUS_MCEERR_AO)) {
-        if (siginfo->si_code == BUS_MCEERR_AR) {
-            /* Fake an Intel architectural Data Load SRAR UCR */
-            mce.status = MCI_STATUS_VAL | MCI_STATUS_UC | MCI_STATUS_EN
-                | MCI_STATUS_MISCV | MCI_STATUS_ADDRV | MCI_STATUS_S
-                | MCI_STATUS_AR | 0x134;
-            mce.misc = (MCM_ADDR_PHYS << 6) | 0xc;
-            mce.mcg_status = MCG_STATUS_MCIP | MCG_STATUS_EIPV;
-        } else {
-            /*
-             * If there is an MCE excpetion being processed, ignore
-             * this SRAO MCE
-             */
-            r = kvm_mce_in_exception(env);
-            if (r == -1)
-                fprintf(stderr, "Failed to get MCE status\n");
-            else if (r)
-                return;
-            /* Fake an Intel architectural Memory scrubbing UCR */
-            mce.status = MCI_STATUS_VAL | MCI_STATUS_UC | MCI_STATUS_EN
-                | MCI_STATUS_MISCV | MCI_STATUS_ADDRV | MCI_STATUS_S
-                | 0xc0;
-            mce.misc = (MCM_ADDR_PHYS << 6) | 0xc;
-            mce.mcg_status = MCG_STATUS_MCIP | MCG_STATUS_RIPV;
-        }
-        if (do_qemu_ram_addr_from_host((void *)siginfo->si_addr, &paddr)) {
-            fprintf(stderr, "Hardware memory error for memory used by "
-                    "QEMU itself instaed of guest system!\n");
-            /* Hope we are lucky for AO MCE */
-            if (siginfo->si_code == BUS_MCEERR_AO)
-                return;
-            else
-                hardware_memory_error();
-        }
-        mce.addr = paddr;
-        r = kvm_set_mce(env, &mce);
-        if (r < 0) {
-            fprintf(stderr, "kvm_set_mce: %s\n", strerror(errno));
-            abort();
-        }
-    } else
-#endif
-    {
-        if (siginfo->si_code == BUS_MCEERR_AO)
-            return;
-        else if (siginfo->si_code == BUS_MCEERR_AR)
-            hardware_memory_error();
-        else
-            sigbus_reraise();
-    }
-}
-
 static void kvm_main_loop_wait(CPUState *env, int timeout)
 {
     struct timespec ts;
@@ -1370,7 +1290,8 @@ static void kvm_main_loop_wait(CPUState *env, int timeout)
 
         switch (r) {
         case SIGBUS:
-            kvm_on_sigbus(env, &siginfo);
+            if (kvm_on_sigbus_vcpu(env, siginfo.si_code, siginfo.si_addr))
+                sigbus_reraise();
             break;
         default:
             break;
@@ -1400,8 +1321,9 @@ static int all_threads_paused(void)
     CPUState *penv = first_cpu;
 
     while (penv) {
-        if (penv->stop)
+        if (penv->stop) {
             return 0;
+        }
         penv = (CPUState *) penv->next_cpu;
     }
 
@@ -1424,8 +1346,9 @@ static void pause_all_threads(void)
         penv = (CPUState *) penv->next_cpu;
     }
 
-    while (!all_threads_paused())
+    while (!all_threads_paused()) {
         qemu_cond_wait(&qemu_pause_cond);
+    }
 }
 
 static void resume_all_threads(void)
@@ -1444,10 +1367,11 @@ static void resume_all_threads(void)
 
 static void kvm_vm_state_change_handler(void *context, int running, int reason)
 {
-    if (running)
+    if (running) {
         resume_all_threads();
-    else
+    } else {
         pause_all_threads();
+    }
 }
 
 static void setup_kernel_sigmask(CPUState *env)
@@ -1532,8 +1456,9 @@ static void *ap_main_loop(void *_env)
     pthread_cond_signal(&qemu_vcpu_cond);
 
     /* and wait for machine initialization */
-    while (!qemu_system_ready)
+    while (!qemu_system_ready) {
         qemu_cond_wait(&qemu_system_cond);
+    }
 
     /* re-initialize cpu_single_env after re-acquiring qemu_mutex */
     cpu_single_env = env;
@@ -1546,8 +1471,9 @@ int kvm_init_vcpu(CPUState *env)
 {
     pthread_create(&env->kvm_cpu_state.thread, NULL, ap_main_loop, env);
 
-    while (env->created == 0)
+    while (env->created == 0) {
         qemu_cond_wait(&qemu_vcpu_cond);
+    }
 
     return 0;
 }
@@ -1593,27 +1519,6 @@ int kvm_init_ap(void)
     return 0;
 }
 
-void qemu_kvm_notify_work(void)
-{
-    /* Write 8 bytes to be compatible with eventfd.  */
-    static uint64_t val = 1;
-    ssize_t ret;
-
-    if (io_thread_fd == -1)
-        return;
-
-    do {
-        ret = write(io_thread_fd, &val, sizeof(val));
-    } while (ret < 0 && errno == EINTR);
-
-    /* EAGAIN is fine in case we have a pipe.  */
-    if (ret < 0 && errno != EAGAIN) {
-         fprintf(stderr, "qemu_kvm_notify_work: write() filed: %s\n",
-                 strerror(errno));
-         exit (1);
-    }
-}
-
 /* If we have signalfd, we mask out the signals we want to handle and then
  * use signalfd to listen for them.  We rely on whatever the current signal
  * handler is to dispatch the signals when we receive them.
@@ -1631,8 +1536,9 @@ static void sigfd_handler(void *opaque)
             len = read(fd, &info, sizeof(info));
         } while (len == -1 && errno == EINTR);
 
-        if (len == -1 && errno == EAGAIN)
+        if (len == -1 && errno == EAGAIN) {
             break;
+        }
 
         if (len != sizeof(info)) {
             printf("read from sigfd returned %zd: %m\n", len);
@@ -1640,50 +1546,23 @@ static void sigfd_handler(void *opaque)
         }
 
         sigaction(info.ssi_signo, NULL, &action);
-        if ((action.sa_flags & SA_SIGINFO) && action.sa_sigaction)
+        if ((action.sa_flags & SA_SIGINFO) && action.sa_sigaction) {
             action.sa_sigaction(info.ssi_signo,
                                 (siginfo_t *)&info, NULL);
-	else if (action.sa_handler)
+        } else if (action.sa_handler) {
             action.sa_handler(info.ssi_signo);
-
+        }
     }
 }
 
-/* Used to break IO thread out of select */
-static void io_thread_wakeup(void *opaque)
-{
-    int fd = (unsigned long) opaque;
-    ssize_t len;
-    char buffer[512];
-
-    /* Drain the notify pipe.  For eventfd, only 8 bytes will be read.  */
-    do {
-        len = read(fd, buffer, sizeof(buffer));
-    } while ((len == -1 && errno == EINTR) || len == sizeof(buffer));
-}
-
 int kvm_main_loop(void)
 {
-    int fds[2];
     sigset_t mask;
     int sigfd;
 
     io_thread = pthread_self();
     qemu_system_ready = 1;
 
-    if (qemu_eventfd(fds) == -1) {
-        fprintf(stderr, "failed to create eventfd\n");
-        return -errno;
-    }
-
-    fcntl(fds[0], F_SETFL, O_NONBLOCK);
-    fcntl(fds[1], F_SETFL, O_NONBLOCK);
-
-    qemu_set_fd_handler2(fds[0], NULL, io_thread_wakeup, NULL,
-                         (void *)(unsigned long) fds[0]);
-
-    io_thread_fd = fds[1];
-
     sigemptyset(&mask);
     sigaddset(&mask, SIGIO);
     sigaddset(&mask, SIGALRM);
@@ -1712,8 +1591,9 @@ int kvm_main_loop(void)
             monitor_protocol_event(QEVENT_SHUTDOWN, NULL);
             if (qemu_no_shutdown()) {
                 vm_stop(0);
-            } else
+            } else {
                 break;
+            }
         } else if (qemu_powerdown_requested()) {
             monitor_protocol_event(QEVENT_POWERDOWN, NULL);
             qemu_irq_raise(qemu_system_powerdown);
@@ -1726,6 +1606,7 @@ int kvm_main_loop(void)
         }
     }
 
+    bdrv_close_all();
     pause_all_threads();
     pthread_mutex_unlock(&qemu_mutex);
 
@@ -1796,6 +1677,18 @@ static int kvm_create_context(void)
     kvm_state->debugregs = kvm_check_extension(kvm_state, KVM_CAP_DEBUGREGS);
 #endif
 
+    kvm_state->xsave = 0;
+#ifdef KVM_CAP_XSAVE
+    kvm_state->xsave = kvm_check_extension(kvm_state, KVM_CAP_XSAVE);
+#endif
+
+    kvm_state->xcrs = 0;
+#ifdef KVM_CAP_XCRS
+    kvm_state->xcrs = kvm_check_extension(kvm_state, KVM_CAP_XCRS);
+#endif
+
+    kvm_state->many_ioeventfds = kvm_check_many_ioeventfds();
+
     kvm_init_ap();
     if (kvm_irqchip) {
         if (!qemu_kvm_has_gsi_routing()) {
@@ -1826,13 +1719,13 @@ int kvm_set_irq(int irq, int level, int *status)
 
 #endif
 
-void kvm_mutex_unlock(void)
+static void kvm_mutex_unlock(void)
 {
     assert(!cpu_single_env);
     pthread_mutex_unlock(&qemu_mutex);
 }
 
-void kvm_mutex_lock(void)
+static void kvm_mutex_lock(void)
 {
     pthread_mutex_lock(&qemu_mutex);
     cpu_single_env = NULL;
@@ -1840,14 +1733,16 @@ void kvm_mutex_lock(void)
 
 void qemu_mutex_unlock_iothread(void)
 {
-    if (kvm_enabled())
+    if (kvm_enabled()) {
         kvm_mutex_unlock();
+    }
 }
 
 void qemu_mutex_lock_iothread(void)
 {
-    if (kvm_enabled())
+    if (kvm_enabled()) {
         kvm_mutex_lock();
+    }
 }
 
 #ifdef CONFIG_KVM_DEVICE_ASSIGNMENT
@@ -1875,8 +1770,9 @@ void kvm_remove_ioperm_data(unsigned long start_port, unsigned long num)
 
 void kvm_ioperm(CPUState *env, void *data)
 {
-    if (kvm_enabled() && qemu_system_ready)
+    if (kvm_enabled() && qemu_system_ready) {
         on_vcpu(env, kvm_arch_do_ioperm, data);
+    }
 }
 
 #endif
@@ -1886,60 +1782,3 @@ int kvm_set_boot_cpu_id(uint32_t id)
     return kvm_set_boot_vcpu_id(kvm_context, id);
 }
 
-#ifdef TARGET_I386
-#ifdef KVM_CAP_MCE
-struct kvm_x86_mce_data {
-    CPUState *env;
-    struct kvm_x86_mce *mce;
-    int abort_on_error;
-};
-
-static void kvm_do_inject_x86_mce(void *_data)
-{
-    struct kvm_x86_mce_data *data = _data;
-    int r;
-
-    /* If there is an MCE excpetion being processed, ignore this SRAO MCE */
-    r = kvm_mce_in_exception(data->env);
-    if (r == -1)
-        fprintf(stderr, "Failed to get MCE status\n");
-    else if (r && !(data->mce->status & MCI_STATUS_AR))
-        return;
-    r = kvm_set_mce(data->env, data->mce);
-    if (r < 0) {
-        perror("kvm_set_mce FAILED");
-        if (data->abort_on_error)
-            abort();
-    }
-}
-#endif
-
-void kvm_inject_x86_mce(CPUState *cenv, int bank, uint64_t status,
-                        uint64_t mcg_status, uint64_t addr, uint64_t misc,
-                        int abort_on_error)
-{
-#ifdef KVM_CAP_MCE
-    struct kvm_x86_mce mce = {
-        .bank = bank,
-        .status = status,
-        .mcg_status = mcg_status,
-        .addr = addr,
-        .misc = misc,
-    };
-    struct kvm_x86_mce_data data = {
-        .env = cenv,
-        .mce = &mce,
-        .abort_on_error = abort_on_error,
-    };
-
-    if (!cenv->mcg_cap) {
-        fprintf(stderr, "MCE support is not enabled!\n");
-        return;
-    }
-    on_vcpu(cenv, kvm_do_inject_x86_mce, &data);
-#else
-    if (abort_on_error)
-        abort();
-#endif
-}
-#endif
